@@ -1,11 +1,15 @@
 import os
 import tempfile
+from abc import ABC, abstractmethod
 from collections import OrderedDict
 from contextlib import contextmanager
 from typing import Any, Dict, Generic, Iterator, List, Optional, Tuple, TypeVar, Union
 
 from encode_utils.connection import Connection
 
+from accession.database.connection import DbSession
+from accession.database.models import Run
+
 K = TypeVar("K")
 V = TypeVar("V")
 T = TypeVar("T")
@@ -76,6 +80,24 @@ class PreferredDefaultFilePatch:
         }
 
 
+class AbstractRecorder(ABC):
+    @abstractmethod
+    def record(self, run: Run) -> None:
+        raise NotImplementedError
+
+
+class Recorder(AbstractRecorder):
+    def __init__(self, use_in_memory_db: bool = False) -> None:
+        if use_in_memory_db:
+            self.db_session = DbSession.with_in_memory_db()
+        else:
+            self.db_session = DbSession.with_home_dir_db_path()
+
+    def record(self, run: Run) -> None:
+        self.db_session.session.add(run)
+        self.db_session.session.commit()
+
+
 def string_to_number(string: str) -> Union[float, str, int]:
     if not isinstance(string, str):
         return string
@@ -127,3 +149,10 @@ def get_api_keys_from_env() -> Tuple[str, str]:
     api_key = os.environ["DCC_API_KEY"]
     secret_key = os.environ["DCC_SECRET_KEY"]
     return api_key, secret_key
+
+
+def get_bucket_and_key_from_s3_uri(s3_uri: str) -> Tuple[str, str]:
+    path_parts = s3_uri.replace("s3://", "").split("/")
+    bucket = path_parts.pop(0)
+    key = "/".join(path_parts)
+    return bucket, key
